# Generated by roxygen2: do not edit by hand

S3method(print,transpiled_call)
export(progressify)
export(progressify_supported_functions)
export(progressify_supported_packages)
export(pz)
importFrom(progressr,progressor)
importFrom(utils,capture.output)
importFrom(utils,getS3method)
importFrom(utils,globalVariables)
importFrom(utils,packageVersion)
importFrom(utils,str)
