#! /usr/bin/env Rscript
## This runs testme test script inst/testme/test-progressify-crossmap.R
## Don't edit - it was autogenerated by inst/testme/deploy.R
progressify:::testme("progressify-crossmap")
